<?php
require_once "fonc_outils.inc.php";
require_once "wpkg_lib.php";
require_once "wpkg_libsql.php";
require_once "wpkg_lib_admin.php";

use GuzzleHttp\Exception\GuzzleException;

/**
 * statut complet d'une machine
 * doit être appelé depuis le serveur local
 * 
 * fusionne 
 *  - les attributs AD
 *  - les atrtributs wpkg (local)
 *  - les attributs log de connexion (local)
 *
 * @param array $config
 * @param string $name : nom de la machine
 * @return array $res
 */
function machine_local_status($config, $name)
{
    $machine = search_machine($config, $name, true);
    $etab = $machine['etab'] ?? 0;
    if (is_local($config)) {
        $wpkg = info_poste_rapport($config, $name)['statut'];
        $machine['objectguid'] = to_guid($machine['objectguid']);
        $machine = array_merge($machine, get_machine_status($config, $machine['cn']), [
            'wpkg' => $wpkg
        ]);
        return $machine;
    } else {
        return $etab;
    }
}

/**
 * statut complet d'une machine
 * peut être appelé depuis le serveur central.
 * Dans ce cas la requète est faite sur le serveur de la machine
 *
 * @param unknown $config
 * @param string $name : nom de la machine
 * @return array $res : ensemble des données
 */
function machine_status($config, $name)
{
    $machine = machine_local_status($config, $name);
    if (! is_array($machine)) {
        require_once(dirname(__FILE__) . '/../vendor/autoload.php');
        $opt = [
            'base_uri' => $config['se4_url'] . "/",
            'cookies' => true
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);

        $path = $machine . "/";
        try {
            $response = $client->get($path . 'api2/status/machine', [
                'query' => [
                    'name' => $name
                ]
            ]);
            $machine = json_decode((string) $response->getBody(), true);
        } catch (GuzzleException $e) {
            // this is delivered each failed request
            // $html .= "erreur : " . $e->getMessage();
            // $html .= "<pre>" . print_r($list, true) . "</pre>";
            $machine = false;
        }
    }
    return $machine;
}

/**
 * statut complet des machines d'un parc
 * doit être appelé depuis le serveur local
 * le parc est a definir avec etab_suffix($uai)
 * 
 * fusionne 
 *  - les attributs AD
 *  - les atrtributs wpkg (local)
 *  - les attributs log de connexion (local)
 *
 * @param unknown $config
 * @param unknown $name : nom du parc
 * @return array $res
 */
function parc_local_status($config, $name)
{
    $res = [];
    $parc = list_members_parc($config, $name, true);
    $wpkg = wpkg_parc_info($config, $name);
    foreach ($parc as $machine) {
        if (isset($machine['cn'])) {
            $machine['objectguid'] = to_guid($machine['objectguid']);
            $machine['code'] = 200;
            $res[] = array_merge($machine, get_machine_status($config, $machine['cn']), [
                'wpkg' => $wpkg[$machine['cn'] ?? []]
            ]);
        }
    }
    return $res;
}

/**
 * statut complet des machines d'un parc
 * $config['etab_ou'] doit correspondre à l'étab du parc
 * 
 * @param unknown $config
 * @param string $name : nom du parc
 * @return array $res : ensemble des données
 */
function parc_status($config, $name)
{
    if (is_local($config)) {
        return parc_local_status($config, $name);
    } else {
        require_once(dirname(__FILE__) . '/../vendor/autoload.php');
        $opt = [
            'base_uri' => $config['se4_url'] . "/",
            'cookies' => true
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);

        if (! empty($config['etab_ou'])) {
            $path = $config['etab_ou'] . "/";
        } else {
            $path = "/";
        }
        try {
            $response = $client->get($path . 'api2/status/parc', [
                'query' => [
                    'name' => $name
                ]
            ]);
            $res = json_decode((string) $response->getBody(), true);
            $res['code'] = $response->getStatusCode();
        } catch (GuzzleException $e) {
            // this is delivered each failed request
            $res['message'] =  $e->getMessage();
            $res['code'] = $e->getCode();
        }
        return $res;
    }
}

/**
 * Demarre, eteint ou reboote un poste en local. 
 *
 * S'execute en local sur le serveur de l'etab
 * 
 * @param array $config
 * @param string $action
 * @param array $machine
 * @param bool $force
 * @param bool $silent
 * @return int $code code http
 */
function start_machine_local($config, $action, array &$machine, $force = false, $silent = false)
{
    $name = strtolower($machine['cn']);

    if ($name == $config['se4fs_name'] || $name == $config['se4ad_name']) {

        return 500;
    }
    $code = 200;
    $ret = false;
    $out = array();
    $msg = "";
    $ip = $machine['iphostnumber'] ?? $name;
    $ping = fping($ip);
    switch ($action) {
        case "status":
            $code = $machine['code'] ?? 200;
            $msg = $machine['message'] ?? "";
            break;
        case "start":
        case "wol":
            if (! $ping || $machine['state'] == "off" || $force) {
                if (empty($machine['networkaddress'])) {
                    log_connexion($config, "", $name, "wol", "startup", "", SAMBAEDU_NO_WOL);
                    $msg .= "On n\'allume pas  <b>$name</b> : elle n'a pas d'adresse mac enregistrée !";
                    $code = 203;
                    break;
                }
                $reseau = get_vlan($config, @$machine['iphostnumber']);
                $msg .= "Mise en marche de la machine  <b>$name</b> : ";
                if ($reseau !== false) {
                    exec("/usr/bin/wakeonlan -i " . long2ip($reseau['broadcast']) . " " . $machine['networkaddress'], $out);
                }
                $msg .= end($out) . "<br>";
                if (isset($config['wol_broadcast'])) {
                    exec("/usr/bin/wakeonlan -i " . $config['wol_broadcast'] . " " . $machine['networkaddress'], $out);
                    $msg .= end($out) . "<br>";
                }
                if (! $ping) {
                    log_connexion($config, "", $name, "wol", "startup");
                    $code = 202;
                } else {
                    $code = 201;
                }
            } else {
                $msg .= "On n\'allume pas  <b>$name</b> : elle est déjà allumée !";
                $code = 200;
            }
            break;

        case "reboot":
            if ($ping && ($machine['state'] != "off" || $force)) {
                $msg .= "On reboote avec l\'action <b>" . $action . "</b> le poste <b>" . $name . "</b> :<br>";
                // if ($machine['state'] != "linux") {
                if ($ping == "windows") {
                    // machine windows
                    if ($silent) {
                        $commande = "/usr/bin/net rpc shutdown --use-kerberos=required -t 2 -f -r -C 'Reboot demande par le serveur sambaEdu' -S " . $name . " >/dev/null 2>&1 &";
                    } else {
                        $commande = "/usr/bin/net rpc shutdown --use-kerberos=required -t 2 -f -r -C 'Reboot demande par le serveur sambaEdu' -S " . $name . " 2>&1";
                    }
                    exec($commande, $out, $ret);
                    if ($ret != 0) {
                        $msg .= "<b>Attention, reboot impossible</b> : " . end($out) . "<br>";
                        log_connexion($config, "", $name, "windows", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                        $msg .= start_machine_local($config, "wol", $machine, true);
                        $code = 203;
                    } else {
                        log_connexion($config, "", $name, "windows", "shutdown");
                        $msg .= end($out);
                        $code = 201;
                    }
                    $msg .= "<br><br>";
                } elseif ($ping == "linux") {
                    // poste linux : ssh...
                    exec("/usr/bin/ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o ConnectTimeout=1 root@" . $machine['cn'] . " shutdown -r now", $out, $ret);
                    $msg .= end($out) . "<br><br>";
                    if ($ret != 0) {
                        $msg .= "On reboote avec l'action <b>" . $action . "</b> le poste <b>" . $name . "</b> :<br>";
                        $msg .= "<b>Attention, reboot impossible</b>, on essaye de la démarrer <br>";
                        log_connexion($config, "", $name, "linux", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                        $msg .= start_machine_local($config, "wol", $machine, true);
                        $code = 203;
                    } else {
                        log_connexion($config, "", $name, "linux", "shutdown");
                        $msg .= "OK<br>\n";
                        $code = 201;
                    }
                }
            } else {
                $msg .= "<b>$name</b>  est éteinte : on l\'allume : <br>";
                $msg .= start_machine_local($config, "wol", $machine, true);
                $code = 201;
            }
            break;

        case "stop":
        case "shutdown":
            if ($force || (isset($machine['user']) && count($machine['user']) == 0)) {
                // if ($machine['state'] != "linux") {
                if ($ping == "windows") {
                    $msg .= "On éteint avec l\'action <b>" . $action . "</b> le poste <b>" . $name . "</b> : <br>";

                    // machine windows
                    if ($silent) {
                        $commande = "/usr/bin/net rpc shutdown --use-kerberos=required -t 30 -f -C \"Arret demande par le serveur sambaEdu\" -S " . $name . " >/dev/null 2>&1 &";
                    } else {
                        $commande = "/usr/bin/net rpc shutdown --use-kerberos=required -t 30 -f -C \"Arret demande par le serveur sambaEdu\" -S " . $name . " 2>&1";
                    }
                    exec($commande, $out, $ret);
                    if ($ret != 0) {
                        $msg .= "<b>Attention, arrêt impossible</b> : " . end($out) . "<br>";
                        log_connexion($config, "", $name, "windows", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                        $code = 203;
                    } else {
                        log_connexion($config, "", $name, "windows", "shutdown");
                        $msg .= end($out);
                        $code = 201;
                    }
                    $msg .= "<br><br>";
                } elseif ($ping == "linux") {
                    $msg .= "On éteint avec l\'action <b>" . $action . "</b> le poste <b>" . $name . "</b> : <br>";

                    // poste linux : ssh...
                    exec("/usr/bin/ssh -i /etc/sambaedu/id_rsa -o StrictHostKeyChecking=no -o ConnectTimeout=1 root@" . $machine['cn'] . " shutdown -h now", $out, $ret);
                    $msg .= end($out) . "<br><br>";
                    if ($ret != 0) {
                        log_connexion($config, "", $name, "linux", "shutdown", "", SAMBAEDU_SHUTDOWN_ERROR);
                        $msg .= "On éteint avec l\'action <b>" . $action . "</b> le poste <b>" . $name . "</b> :<br>";
                        $msg .= "<b>Attention, arrêt impossible</b>, la machine est injoignable ! <br>";
                    } else {
                        log_connexion($config, "", $name, "linux", "shutdown");
                        $msg .= "OK<br>\n";
                    }
                } else {
                    $msg .= "action <b>" . $action . "</b> le poste <b>" . $name . "</b> est déjà eteint <br>";
                    $code = 200;
                }
            } else {
                $msg .= "On n'\'éteint pas le <b> le poste <b>" . $name . "</b> : un utilisateur est connecté<br>";
                $code = 203;
            }
            break;
    }
    $machine['code'] = $code;
    $machine['message'] = $msg;
    return $code;
}

/**
 * demarrage d'une machine
 * peut être appelé depuis le serveur central.
 * Dans ce cas la requète est faite sur le serveur de la machine
 *
 * @param unknown $config
 * @param string $action
 * @param array &$machine
 * @param bool $force
 * @return array $code : code http
 *  
 */
function start_machine($config, $action, &$machine, $force = false)
{
    if (is_local($config)) {
        start_machine_local($config, $action, $machine, $force);
    } else {
        require_once(dirname(__FILE__) . '/../vendor/autoload.php');
        $opt = [
            'base_uri' => $config['se4_url'] . "/",
            'cookies' => true
        ];
        curl_proxy_options($config, $opt);
        $client = new GuzzleHttp\Client($opt);
        if (!empty($machine['etab'])) {
            $path = $machine['etab'] . "/";
        } else {
            $path = "/";
        }
        try {
            $response = $client->post($path . 'api2/action/machine', [
                'form_params' => [
                    'machine' => json_encode($machine, JSON_INVALID_UTF8_IGNORE),
                    'action' => $action,
                    'force' => $force
                ]
            ]);
            $machine = json_decode((string) $response->getBody(), true);
            $machine['code'] = $response->getStatusCode();
        } catch (GuzzleException $e) {
            // this is delivered each failed request
            $machine['message'] .= $e->getMessage();
            $machine['code'] = $e->getCode();
        }
    }
    return $machine['code'];
}


/**
 * fonction permettant l'appel asynchrone du statut et des actions sur les postes passées dans $list
 * les requètes seront faites en // sur les serveurs d'etab des machines
 *
 * @param unknown $config
 * @param string $action
 *            (status|start|stop|reboot) @TODO a étendre ?
 * @param array &$list
 *            liste AD des machines, statut mis à jour par la fonction. Peut correspondre au résultat de parc_status(), mais peut aussi être une liste de machines individuelles
 * @param array &$reste
 *            machines non traitées ( pas de réponse du serveur ?)
 */
function start_async($config, $action, &$list, &$reste = [])
{
    // appel en parallèle de chaque machine, sauf pour celles qui sont ok en cache.
    require_once(dirname(__FILE__) . '/../vendor/autoload.php');
    $opt = [
        'base_uri' => $config['se4_url'] . "/",
        'cookies' => true
    ];
    curl_proxy_options($config, $opt);
    $client = new GuzzleHttp\Client($opt);
    $requests = function ($list, $action, $reste) use ($client) {
        foreach ($list as $k => $machine) {
            if (! isset($machine['code']) || $machine['code'] != 200) {
                yield $k => function () use ($client, $machine, $action) {
                    $config = [];
                    $config = get_config($config, true);
                    if (! empty($machine['etab'])) {
                        $path = $machine['etab'] . "/";
                    } else {
                        $path = "/";
                    }
                    return $client->postAsync($path . 'api2/action/machine', [
                        'form_params' => [
                            'machine' => htmlspecialchars(json_encode($machine, JSON_INVALID_UTF8_IGNORE)),
                            'action' => $action,
                            'force' => true
                        ]
                    ]);
                };
            }
        }
    };

    $pool = new GuzzleHttp\Pool($client, $requests($list, $action, $reste), [
        'concurrency' => 100,
        'fulfilled' => function (GuzzleHttp\Psr7\Response $response, $index) use (&$list, &$reste) {
            // this is delivered each successful response
            $res = json_decode((string) $response->getBody(), true);
            $res['code'] = $response->getStatusCode();
            $list[$index] = $res;
        },
        'rejected' => function (GuzzleException $reason, $index) use (&$list, &$reste) {
            // this is delivered each failed request
            //echo "erreur : " . $reason->getMessage();
            // echo "<pre>" . print_r($list, true) . "</pre>";
            //$reste[] = $list[$index];
            //unset($list[$index]);
            $res['message'] = $reason->getMessage();
            $res['code'] = $reason->getCode();
            $list[$index] = $res;
        }
    ]);

    // Initiate the transfers and create a promise
    $promise = $pool->promise();

    // Force the pool of requests to complete.
    $promise->wait();
    // $res = $parc;
}

/**
 * Demarre, eteint ou reboote un parc
 *
 * @Parametres action: wol, reboot, shutdown
 *             parc : nom du parc
 *
 * @return
 *
 */
function start_parc($config, $action, $parc, $force = false, $silent = true)
{
    $liste = list_members_parc($config, $parc, true);
    sort($liste);
    $ret = "";
    foreach ($liste as $value) {
        $ret .= start_machine($config, $action, $value, $force, $silent) . "<br>\n";
        // usleep(300000);
    }
    return $ret;
}
/**
 * genere le formulaire de selection d'un parc
 *
 * @param array $config
 * @param array $list_parcs
 * @param array $delegations
 * @return string $html
 */
function form_select_parc($config, $list_parcs, $delegations, $parc = "")
{
    if (count($list_parcs) > 0) {
        $html = "<h3> Sélectionner un parc :</h3>";
        $html .= "<CENTER>";
        $html .= "<FORM id=\"select\" method=\"post\" action=\"action_parc.php\">\n";
        $html .= "<SELECT NAME=\"parc\" SIZE=\"1\" onchange=submit()>";
        $html .= "<option value=\"SELECTIONNER\">Sélectionner</option>";
        $nb_etab = count(activated_etabs($config));
        foreach ($list_parcs as $value_tmp) {
            if (! empty($value_tmp["samaccountname"]) && (empty($delegations) || in_array($value_tmp["gdn"], array_column($delegations, "parcdn")))) {
                $html .= "<option value=\"" . $value_tmp["samaccountname"] . "\"";
                if ($parc == $value_tmp["samaccountname"]) {
                    $html .= " selected";
                    $etab = $value_tmp["etab"];
                }
                $nom_etab = "";
                if ($nb_etab > 1) { // && $etab != $value_tmp["etab"]) {
                    $nom_etab = " " . etab_to_name($config, $value_tmp["etab"]);
                }
                $html .= ">" . $value_tmp["name"] . " " . " de type " . $value_tmp["type"] . $nom_etab . "\n";
                $html .= "</option>";
            }
        }
        // }

        $html .= "</SELECT>&nbsp;&nbsp;\n";
        $html .= "<img onmouseout=\"UnTip();\" onmouseover=\"Tip('Sélectionner un parc',WIDTH,250,SHADOW,true,DURATION,5000);\" name=\"action_image2\"  src=\"../elements/images/system-help.png\" alt=\"help\"> ";
        // $html .= "<input type=\"submit\" value=\"Valider\">\n";
        $html .= "</FORM>\n";
        $html .= "</CENTER>\n";
        $html .= "<br>";
    } else {
        $html = "<br><center>";
        $html .= "Il n'existe pas encore de parc";
        $html .= "</center>\n";
    }
    return $html;
}
/**
 * genere le formulaire des actions applicables sur un parc
 *
 * @param array $config
 * @param string $parc
 * @return string $html
 */
function form_actions_parc($config, $parc)
{
    $html = "<td>";
    $html .= "<form id=\"stop\" action=\"action_parc.php\" method=\"post\">\n";
    $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
    $html .= "<input type=\"hidden\" name=\"action\" value=\"stop\" />";
    $html .= "<input type=\"submit\" value=\"Eteindre tous les postes\" onclick=\"if (window.confirm('Etes-vous sur de vouloir éteindre le parc ?')) {return true;} else {return false;}\" />";
    $html .= "</form>\n";
    $html .= "</td>";

    $html .= "<td>";
    $html .= "<form id=\"start\" action=\"action_parc.php\" method=\"post\">\n";
    $html .= "<input type=\"hidden\" name=\"action_poste\" value=\"wol\" />";
    $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
    $html .= "<input type=\"hidden\" name=\"action\" value=\"start\" />";
    $html .= "<input type=\"submit\" value=\"Allumer tous les postes\" />";
    $html .= "</form>\n";
    $html .= "</td>";

    // ===================================
    $html .= "<td>";
    $html .= "<form id=\"reboot\" action=\"action_parc.php\" method=\"post\">\n";
    $html .= "<input type=\"hidden\" name=\"action_poste\" value=\"reboot\" />";
    $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
    $html .= "<input type=\"hidden\" name=\"action\" value=\"reboot\" />";
    $html .= "<input type=\"submit\" value=\"Rebooter tous les postes\" onclick=\"if (window.confirm('Etes-vous sur de vouloir rebooter le parc ?')) {return true;} else {return false;}\" />";
    $html .= "</form>\n";
    $html .= "</td>";
    // ===================================
    // Test le niveau de delegation
    // manage ou view
    // Si manage on peut aller sur programmer
    $niveau_delegation = type_delegation($config, $parc);
    if ($niveau_delegation & ~SE_COMPUTER_VIEW) {
        $html .= "<td>";
        $html .= "<form id=\"program\" action=\"action_parc.php\" method=\"post\">\n";
        $html .= "<input type=\"hidden\" name=\"programmation\" value=\"check\" />";
        $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
        $html .= "<input type=\"hidden\" name=\"action\" value=\"timing\" />";
        $html .= "<input type=\"submit\" value=\"Programmer l'allumage\" />";
        $html .= "</form>\n";
        $html .= "</td></tr><tr>\n";
        $html .= "<td>";
        $html .= "<form id=\"install_l\" action=\"action_parc.php\" method=\"post\">\n";
        $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
        $html .= "<input type=\"hidden\" name=\"type\" value=\"install\" />";
        $html .= "<input type=\"submit\" value=\"Installer Linux/Windows\" />";
        $html .= "</form>\n";
        $html .= "</td>\n";
        $html .= "<td><form id=\"renomme\" action=\"lance_action.php\" method=\"post\">\n";
        $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
        $html .= "<input type=\"hidden\" name=\"type\" value=\"renomme\" />";
        $html .= "<input type=\"submit\" value=\"Renommer les postes\" />";
        $html .= "</form>\n";
        $html .= "</td>\n";
        $html .= "<td>";
        $html .= "<form id=\"wpkg\" action=\"lance_action.php\" method=\"post\">\n";
        $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
        $html .= "<input type=\"hidden\" name=\"type\" value=\"wpkg\" />";
        $html .= "<input type=\"submit\" value=\"Lancer Wpkg en mode interactif\" />";
        $html .= "</form>\n";
        $html .= "</td>\n";
        $html .= "<td>";
        $html .= "<form id=\"clone\" action=\"lance_action.php\" method=\"post\">\n";
        $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
        $html .= "<input type=\"hidden\" name=\"type\" value=\"clonage\" />";
        $html .= "<input type=\"hidden\" name=\"action\" value=\"new\" />";
        $html .= "<input type=\"submit\" value=\"Cloner\" />";
        $html .= "</form></td>\n";
    }

    if (count(list_members_parc($config, "serveurs_ltsp"), false) > 0) {
        $html .= "<td>";
        $html .= "<form id=\"ltsp\" action=\"lance_action.php\" method=\"post\">\n";
        $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
        $html .= "<input type=\"hidden\" name=\"type\" value=\"ltsp\" />";
        $html .= "<input type=\"submit\" value=\"Demarrer les postes en client LTSP\" />";
        $html .= "</form>";
        $html .= "</td></tr>\n";
    }

    $html .= "<tr><td colspan=\"2\" align=\"center\">";
    $html .= "<form id=\"show\" action=\"show_parc.php\" method=\"post\">\n";
    $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
    $html .= "<input type=\"submit\" value=\"Lister le parc\" />";
    $html .= "</form>\n";
    $html .= "</td>\n";
    // $html .= "<td colspan=\"3\" align=\"center\"><form action=\"action_parc.php\" method=\"post\">\n";
    $html .= "<td colspan=\"2\" align=\"center\"><form id=\"action\" action=\"action_parc.php\" method=\"post\">\n";
    $html .= "<input type=\"hidden\" name=\"action_poste\" value=\"check\" />";
    $html .= "<input type=\"hidden\" name=\"parc\" value=\"$parc\" />";
    $html .= "<input type=\"hidden\" name=\"action\" value=\"detail\" />";
    $html .= "<input type=\"submit\" value=\"Rafraîchir la page\" />";
    $html .= "</form>\n";

    $html .= "</td></tr></table>\n";
    return $html;
}
/**
 * affiche la liste des machines
 *
 * @param array $config
 * @param string $parc
 * @param array $mp
 * @param boolean $refresh
 * @return string $html
 */
function form_machines_parc($config, string $parc, array $mp)
{
    $refresh = false;
    $html = "\n<br>\n<CENTER>\n";
    //    $html .= "<form id=\"action_parc\" enctype=\"multipart/form-data\" action=\"action_parc.php\" method=\"post\">\n";
    $html .= "<form id=\"action_parc\" action=\"action_parc.php\" method=\"post\">\n";

    $actions = ["detail" => "Rafraichir", "start" => "Allumer", "stop" => "Eteindre", "reboot" => "redémarrer"];
    $html .= "<select name=action >";
    foreach ($actions as $a => $texte) {
        $html .= "<option value=\"" . $a . "\" ";
        $html .= ">" . $texte . "</option>";
    }
    $html .= "</select>";
    $html .= "<input name=go value=\"Valider\" type=submit>";

    $html .= "<input name=parc value=\"" . $parc . "\" type=hidden>";

    $html .= "<table class='table-bordered' width=\"80%\">\n<tr class=menuheader style=\"height: 30\">\n";
    $html .= "<td class='menuheader' align=\"center\">Tout selectionner<br>";
    $html .= '<input type="checkbox" id="select-all">';
    $html .= "</td>";
    $html .= "<td align=\"center\">OS</td>\n";
    $html .= "<td align=\"center\">STATIONS CONCERNEES</td>\n";
    $html .= "<td align=\"center\">ADRESSE IP</td>\n";
    $html .= "<td align=\"center\">ETAT</td>\n";
    $html .= "<td align=\"center\">CONNEXION</td>\n";
    $html .= "<td align=\"center\">CONTROLE</td>\n";
    $html .= "<td align=\"center\">Wpkg</td>\n";
    $html .= "<td align=\"center\">Erreurs</td></tr>\n";
    $html .= "</tr>\n";
    $html .= "<script>
            function openInNewWindow(event, link) {
                event.preventDefault(); // Empêche l'ouverture par défaut
                window.open(link.href, '_blank', 'noopener,noreferrer,width=1920,height=1080');
            }
            </script>";


    if (is_array($mp) && count($mp) > 0) {
        foreach ($mp as $index => $machine) {
            if (isset($machine['cn'])) {
                $mpenc = ldap_hostname($config, $machine['cn']);
                $mp_en_cours = urldecode($mpenc);
                $resultat = get_os_icon($config, $machine);
                // élimination des imprimantes de la liste
                if ($resultat['os'] == "imprimante" || $resultat['os'] == "equipement") {
                    // unset ($mp[$loop]);
                    continue;
                }
                $from = urlencode("../parcs/action_parc.php?action=detail&parc=" . $parc);

                // Affichage du tableau
                $html .= "<tr>\n";
                // Affichage coche
                $html .= "<td align=\"center\"><INPUT type=\"checkbox\" name=\"indexes[]\" id=\"machine_$index\"  value=\"" . $index . "\">";
                $html .= "</td>\n";
                // Affichage de l'icone informatique
                $html .= "<td align=\"center\">\n";
                $machine['state'] = $machine['state'] ?? "";
                $iphost = $machine['iphostnumber'] ?? "";
                $machine['networkaddress'] = $machine['networkaddress'] ?? "";
                $systemid = $iphost;
                $ip_link = "";
                $html .= "<img style=\"border: 0px solid ;\" src=\"../elements/images/" . $resultat['icone'] . "\" title=\"" . $resultat['os'] . " - " . $iphost . "\" alt=\"" . $resultat['os'] . "\" WIDTH=20 HEIGHT=20 onclick=\"popuprecherche('../ocsreports/machine.php?systemid=$systemid','popuprecherche','scrollbars=yes,width=500,height=550');\">";

                // $html .= "<img style=\"border: 0px solid ;\" src=\"../elements/images/computer.png\" onclick=\"popuprecherche('../ocsreports/machine.php?systemid=$systemid','popuprecherche','scrollbars=yes,width=500,height=550');\" title=\"Station\" alt=\"Station\"></td>\n";
                $html .= "<td align=center ><a href=\"cherche_machine.php?mpenc=" . $machine['cn'] . "&from=$from\">$mp_en_cours</a></td>\n";
                if ($machine['state'] == "reservation") {
                    $ip_link = "ip réservée : " . $iphost;
                } elseif ($machine['state'] == "active") {
                    $ip_link = "<a href=action_machine.php?retour=action_parc&action=reservation&machine=" . $machine['cn'] . "&mac=" . $machine['networkaddress'] . " onmouseout=\"UnTip();\" onmouseover=\"Tip('Réserver une ip pour ce poste.',WIDTH,250,SHADOW,true,DURATION,5000);\"> ip dynamique : $iphost</a>";
                } elseif ($machine['state'] == "dns") {
                    $ip_link = "ip dns : " . $iphost;
                } else {
                    $ip_link = "ip " . $iphost . " : " . $machine['state'];
                }
                $html .= "<td align=center>$ip_link</td>\n";
                $html .= "<td align=center>\n";
                $message = $machine['message'] ?? "";
                $infobulle = "onmouseout=\"UnTip();\" onmouseover=\"Tip('Etat la machine : " . $machine['state'] . ",<br> code " . $machine['code'] . "<br>" . $message . "',WIDTH,250,SHADOW,true,DURATION,5000);\"";
                // $etat
                // $html .= "<div id='divip$loop'><img src=\"../elements/images/spinner.gif\"></img></div>\n";
                switch (@$machine['state']) {
                    case "windows":
                    case "linux":
                        $image = "../elements/images/enabled.png";
                        break;
                    case "wol":
                    case "ipxe":
                         $machine['code'] = 201;
                        break;
                    default:
                        $image = "../elements/images/disabled.png";
                }
                if ($machine['code'] != 200){
                    $image = "../elements/images/spinner.gif";
                    $refresh = true;
                }
                $html .= "<input name=\"start[]\" type=image src=\"" . $image . "\" " . $infobulle . ">";

                /*
             * $html .= "<script type='text/javascript'>
             * // <![CDATA[
             * new Ajax.Updater($('divip$loop'),'parcs_ajax_lib.php',{method: 'post', parameters: '?ip=$iphost&parc=$parc&nom_machine=" . $machine['cn'] . "&mode=ping_ip' });
             * //]]>
             * </script>\n";
             */
                $html .= "</td>\n";
                $html .= "<td align=center>\n";
                // $etat_session
                // if (fping($iphost)){
                // $html .= "<div id='divsession$loop'><img src=\"../elements/images/spinner.gif\"></img></div>\n";
                $html .= "<a target=\"main\" href=\"show_histo.php?selectionne=2&amp;mpenc=" . $machine['cn'] . "&from=$from\">";
                if (empty($machine['user'][0]['name'])) {
                    $html .= "<img type=\"image\" src=\"../elements/images/disabled.png\">\n";
                } elseif ($machine['user'][0]['name'] == $config['se4install_name']) {
                    $texte = $machine['user'][0]['name'] . " est actuellement connecté sur ce poste";
                    $html .= "<img onmouseout=\"UnTip();\" onmouseover=\"Tip('" . $texte . "',WIDTH,250,SHADOW,true,DURATION,5000);\" src=\"../elements/images/travaux2.png\" border=\"0\" />";
                } elseif ($machine['user'][0]['name'] == "wpkg") {
                    $texte = $machine['user'][0]['name'] . " est actuellement connecté sur ce poste";
                    $html .= "<img onmouseout=\"UnTip();\" onmouseover=\"Tip('" . $texte . "',WIDTH,250,SHADOW,true,DURATION,5000);\" src=\"../elements/images/travaux2.png\" border=\"0\" />";
                } else {
                    $texte = $machine['user'][0]['name'] . " est actuellement connecté sur ce poste";
                    $html .= "<img onmouseout=\"UnTip();\" onmouseover=\"Tip('" . $texte . "',WIDTH,250,SHADOW,true,DURATION,5000);\" src=\"../elements/images/enabled.png\" border=\"0\" />";
                }

                /*
             * $html .= "<script type='text/javascript'>
             * // <![CDATA[
             * new Ajax.Updater($('divsession$loop'),'parcs_ajax_lib.php',{method: 'post', parameters: '?nom_machine=" . $machine['cn'] . "&mode=session'});
             * //]]>
             * </script>\n";
             */
                /*
             * } else {
             * $ret = "<span onmouseout=\"UnTip();\" onmouseover=\"Tip('Echec du ping',WIDTH,250,SHADOW,true,DURATION,5000);\">" . "<img type=\"image\" border=\"0\" title=\"Echec du ping\" src=\"../elements/images/disabled.png\">" . "</span>\n";
             * $html .= ($ret);
             * }
             */
                $html .= "</a></td>\n";
                $html .= "<td align=\"center\">";
                // if (fping($iphost)){
                //$url = "rdp.php?machine=" . $machine['cn'] . "&mode=direct&type=rdp&from=" . $from;
                $url = create_remote_token($config, $machine, "rdp", $config['login']);
                $html .= "<a href=\"" . $url . "\" onclick=\"openInNewWindow(event, this)\"><img src=\"../elements/images/rdp-small.png\" alt=\"Accès distant...\"></a>";
                // $html .= "<div id='divtsvnc$loop'><img src=\"../elements/images/spinner.gif\"></img></div>\n";
                /*
             * $html .= "<script type='text/javascript'>
             * // <![CDATA[
             * new Ajax.Updater($('divtsvnc$loop'),'parcs_ajax_lib.php',{method: 'post', parameters: '?ip=$iphost&mode=ts_vnc'});
             * //]]>
             * </script>\n";
             */
                /*
             * } else {
             * $ret = "<span onmouseout=\"UnTip();\" onmouseover=\"Tip('Echec du ping',WIDTH,250,SHADOW,true,DURATION,5000);\">" . "<img type=\"image\" border=\"0\" title=\"Echec du ping\" src=\"../elements/images/disabled.png\">" . "</span>\n";
             * $html .= ($ret);
             * }
             */
                $html .= "</td>";
                $html .= "<td align=\"center\">";
                $r = $machine['error'] & SAMBAEDU_WPKG_RUNNING;
                $html .= "<a target=\"main\" href=\"../wpkg/poste_statuts.php?id_host=" . $machine['cn'] . "&from=$from\">";
                $w = $machine['wpkg'] ?? "";

                if ($r) {
                    $texte = "<li>" . implode("<li>", error_message($r));
                    $img = "../elements/images/spinner.gif";
                } else {
                    switch ($w) {
                        case 0:
                            $texte = "Applications OK";
                            $img = "../elements/images/enabled.png";
                            break;
                        case 1:
                            $texte = "Au moins une mise à jour est en attente";
                            $img = "../elements/images/warning.png";
                            break;
                        case 2:
                            $texte = "Au moins une application n&lsquo;est pas installée";
                            $img = "../elements/images/dialog-warning.png";
                            break;
                        default:
                            $texte = "Pas de rapport wpkg";
                            $img = "../elements/images/help.png";
                            break;
                    }
                }
                $html .= "<img onmouseout=\"UnTip();\" onmouseover=\"Tip('" . $texte . "',WIDTH,250,SHADOW,true,DURATION,5000);\" type=\"image\" src=\"" . $img . "\" border=\"0\" title=\"" . $machine['cn'] . "\" title=\"" . $texte . "\" />";
                $html .= "</a></td>\n";
                $html .= "</td>\n";
                unset($texte);
                $html .= "<td align=\"center\">";
                $r = $machine['error'] & ~SAMBAEDU_WPKG_RUNNING;
                $html .= "<a target=\"main\" href=\"show_histo.php?selectionne=2&amp;mpenc=" . $machine['cn'] . "&from=$from\">";
                if ($r > 0) {
                    $texte = "<li>" . implode("<li>", error_message($r));
                    $html .= "<img onmouseout=\"UnTip();\" onmouseover=\"Tip('" . $texte . "',WIDTH,250,SHADOW,true,DURATION,5000);\" src=\"../elements/images/dialog-warning.png\" border=\"0\" />";
                } else {
                    $html .= "<img type=\"image\" src=\"../elements/images/enabled.png\" border=\"0\" title=\"" . $machine['cn'] . "\" title=\"" . $machine['cn'] . "\" />";
                }
                if ($machine['error'] & SAMBAEDU_SLOW_NET_ERROR) {
                    $html .= "<img type=\"image\" src=\"../elements/images/snail.png\" border=\"0\" title=\"" . $machine['cn'] . "\" title=\"" . $machine['cn'] . "\" />";
                }
                $html .= "</a></td>\n";

                $html .= "</tr>\n";
            }
        }
    }
    $html .= "</table>\n";
    $html .= "</center>\n";
    $html .= "<br>";
    $html .= "<input name=\"mp\" type=\"hidden\" value=\"" . htmlspecialchars(json_encode($mp, JSON_INVALID_UTF8_IGNORE)) . "\">";
    $html .= "</form>\n";
    if ($refresh) {
        require_once "actions.inc.php";
        $html .= refresh_page("action", 30);
    }

    return $html;
}
/**
 * affichage de la page permettant d'agir sur un parc
 *
 * @param array $config
 * @param string $html
 * @return void
 */
function action_parc_html($config, &$html)
{
    $etab = $_SESSION['etab'] ?? $config['etab_ou'] ?? "";
    $html .= "<script type=\"text/javascript\" src=\"/elements/js/wz_tooltip_new.js\"></script>";
    // Verifie les droits
    if (have_right_or_delegation($config, SE_COMPUTER_VIEW)) {

        $html .= "<h1>Actions sur les stations</h1>";

        /* * *********************** Declaration des variables *********************************** */
        $action = $_POST['action'] ?? $_GET['action'] ?? "detail";
        $parc = $_POST['parc'] ?? $_GET['parc'] ?? "";
        $indexes = $_POST['indexes'] ?? $_GET['indexes'] ?? false;
        $action_poste = $_POST['action_poste'] ?? $_GET['action_poste'] ?? "";
        if ($action_poste == "check") {
            $refresh = 1;
        } else {
            $refresh = 0;
        }
        $nomjour = date("l");
        if ($action == "choix_time") {
            $action = "detail";
        }
        $delegations = list_delegations($config);
        if (empty($parc) || $parc == "SELECTIONNER") {
            $list_parcs = search_parcs($config, "*");
            $html .= form_select_parc($config, $list_parcs, $delegations, $parc);
            $mp = [];
        } else {
            if (!empty($_POST['mp'])) {
                $mp = json_decode(htmlspecialchars_decode($_POST['mp']), true);
            }
            if (!is_array($mp)) {
                $mp =  parc_status($config, $parc);
            }
        }
        switch ($action) {
            case "reboot":
            case "wol":
            case "stop":
            case "start":
                //$html .= "<pre>" . print_r($indexes, true) . "</pre>";
                if ($indexes !== false) {
                    foreach ($indexes as $index) {
                        if (isset($mp[$index])) {
                            //$list[$index] = $mp[$index];
                            unset($mp[$index]['code']);
                            //$l = $mp[$index];
                            //start_machine($config, $action, $l, true);
                            //$mp[$index] = $l;
                        }
                    }
                    start_async($config, $action, $mp, $html);
                    //$html .= "<pre>" . print_r($list, true) . "</pre>";
                    //foreach ($indexes as $index) {
                    //    $mp[$index] = $list[$index];
                    //}
                    //                  $html .= "<pre>" . print_r($list, true) . "</pre>";
                }
            case "detail":
                //               if (is_array($mp) && count($mp) > 0) {
                $html .= "<CENTER><table align=center><tr>\n";
                $html .= "<td colspan=3>";
                // @TODO : choisir et allumer une machine libre sur le parc et la proposer en veyon-master
                //
                //$url = create_remote_token($config, search_machine($config, $config['remote_admin_machine'] . etab_suffix($etab)), "master", $config['login']);
                //$html .= "<a href=\"" . $url . "\" onclick=\"openInNewWindow(event, this)\"><img src=\"../elements/images/rdp-small.png\" alt=\"Accès distant...\">Contrôle a distance du parc</a>";
                $html .= "</td></tr><tr>\n";
                // affichage des actions de masse
                //                    $html .= form_actions_parc($config, $parc);
                $html .= "</center>\n";
                // affichage tableau des machines
                $html .= form_machines_parc($config, $parc, $mp, $refresh);
                //               }
                break;
            default:
                $html .= "coucou toi";
                break;
        }
    }
}

function rdp_html($config, &$html)
{
    require_once "remote.inc.php";
    // mode de connexion
    $mode = $_GET['mode'] ?? "list";
    $type = $_GET['type'] ?? "rdp";
    switch ($mode) {
        case "direct":
            // redirection vers l'url de guacamole avec un token
            $url = create_remote_token($config, $_GET['machine'], $type, $config['login']);
            header("Location: " . $url);
            exit();
            break;
        default:
            $login = $config['login'];
            // if (have_right($config, SE_COMPUTER_ADMIN)) {
            $html .= "<h1>accès distant</h1>\n";
            if (! file_exists("/etc/sambaedu/sambaedu.conf.d/guacamole.conf")) {
                if ($config['login'] == "admin") {
                    $html .= "Cet outil semble manquant sur le serveur !";
                    $html .= "<ul>\n";
                    $html .= "<li>Vous devez installer le module sambaedu-guacamole afin d'utiliser l'accès distant</li>\n";
                    $html .= "</ul>\n";
                } else {
                    $html .= "Fonctionalité non disponible";
                }
            } else {
                $html .= display_remote_list($config, $login);
            }
    }
}
/**
 * page déclenchant une action sur une machine.
 * 
 * selon les actions cette page pourra soit faire un simple redirect, soit afficher une page
 *
 * @param [type] $config
 * @param [type] $html
 * @return void
 */
function action_machine_html($config, &$html)
{

    // ***************Definition des droits de lecture et aide en ligne
    $login = $config['login'];
    // Verifie les droits
    if (! have_right_or_delegation($config, SE_COMPUTER_VIEW)) {
        exit();
    }
    $acces_restreint = false;
    // *****************cas des parcs delegues***********************************
    if (! have_right_or_delegation($config, SE_COMPUTER_ADMIN, $login)) {
        $html .= "<h3>Votre délégation a été prise en compte pour l'affichage de cette page.</h3>";
        $acces_restreint = true;

        $list_delegate = list_delegations($config);
        $html .= "<ul>";
        foreach ($list_delegate as $info_parc_delegate) {
            $html .= "<li>$info_parc_delegate</li>";
        }
        $html .= "</ul>";
    }

    /**
     * *********************** Declaration des variables ***********************************
     */
    // action peut etre shutdown ou wol
    $action = $_POST['action'] ?? $_GET['action'] ?? "";
    $ip = $_POST['ip'] ?? $_GET['ip'] ?? "";
    $m = $_POST['machine'] ?? $_GET['machine'] ?? "";
    $machine = machine_status($config, $m);
    $retour = $_POST['retour'] ?? $_GET['retour'] ?? "show_parc";
    $parc = $_GET['parc'] ?? "";
    $content = "";

    if ($acces_restreint) {
        if (! have_delegation($config, $parc, SE_COMPUTER_VIEW)) {
            exit();
        }
    }

    switch ($action) {
        case "shutdown":
            if (! empty($machine)) {
                $content .= "<HEAD>";
                $content .= "<META HTTP-EQUIV=\"refresh\" CONTENT=\"5; URL=$retour.php?parc=$parc&action_poste=check&action=choix_time \">";
                $content .= "</HEAD>";

                $content .= "<h1>Arrêt de(s) machine(s)</h1>";
                $content .= "Commandes prises en compte !<br>";
                $content .= start_machine($config, "shutdown", $machine, true);
                $content .= "<h3>Arret lancé pour le poste" . $machine['cn'] . "</h3>";
                $content .= "<font color=#FF0000>Un temps d'attente de plus une minute est nécéssaire pour voir le résultat dans l'interface</font><br>";
            } else {
                $content .= "Vous devez choisir un parc";
            }

            break;

        case "wol":
            if (! empty($machine)) {
                $content .= "<HEAD>";
                $content .= "<META HTTP-EQUIV=\"refresh\" CONTENT=\"5; URL=$retour.php?parc=$parc&action_poste=check&action=choix_time \">";
                $content .= "</HEAD>";

                $content .= "<h1>Démarrage de(s) machine(s)</h1>";

                $content .= "Commandes prises en compte !<br>";

                $content .= "<h3>Démarrage effectué pour le poste" . $machine['cn'] . " (Ne concerne que les machines equipées du syst&#232me 'wake on lan')</h3>";
                $content .= start_machine($config, "wol", $machine, true);
                $content .= "<font color=#FF0000>Un temps d'attente de plus une minute est nécéssaire pour voir le résultat dans l'interface</font><br>";
            } else {
                $content .= "Vous devez choisir une machine";
            }

            break;

        case "reboot":
            if (! empty($machine)) {
                $content .= "<HEAD>";
                $content .= "<META HTTP-EQUIV=\"refresh\" CONTENT=\"5; URL=$retour.php?parc=$parc&action_poste=check&action=choix_time \">";
                $content .= "</HEAD>";

                $content .= "<h1>Redémarrage de(s) machine(s)</h1>";

                $content .= "Commandes prises en compte !<br>";

                $content .= "<h3>Redémarrage effectué pour le poste" . $machine['cn'] . " (Ne concerne que les machines equipées du syst&#232me 'wake on lan')</h3>";
                $content .= start_machine($config, "reboot", $machine, true);
                $content .= "<font color=#FF0000>Un temps d'attente de plus une minute est nécéssaire pour voir le résultat dans l'interface</font><br>";
            } else {
                $content .= "Vous devez choisir une machine";
            }

            break;

        case "reservation":
            if (! empty($machine)) {
                $content .= "<HEAD>";
                $content .= "<META HTTP-EQUIV=\"refresh\" CONTENT=\"5; URL=$retour.php?parc=$parc&action_poste=check&action=choix_time \">";
                $content .= "</HEAD>";

                $content .= "<HEAD>";
                $content .= "<META HTTP-EQUIV=\"refresh\" CONTENT=\"5; URL=$retour.php?parc=$parc&action_poste=check&action=choix_time \">";
                $content .= "</HEAD>";
                $content .= start_poste($config, "wol", $machine, true);
                if (set_dhcp_reservation($config, $machine['cn'], $ip)) {
                    $content .= "reservation ";


                    $content .= "reservation pour" . $machine['cn'] . "impossible<br>\n";
                }
            }
            break;
        case "dynamique":
            if (! empty($machine)) {
                $content .= "<HEAD>";
                $content .= "<META HTTP-EQUIV=\"refresh\" CONTENT=\"5; URL=$retour.php?parc=$parc&action_poste=check&action=choix_time \">";
                $content .= "</HEAD>";

                $content .= "<HEAD>";
                $content .= "<META HTTP-EQUIV=\"refresh\" CONTENT=\"5; URL=$retour.php?parc=$parc&action_poste=check&action=choix_time \">";
                $content .= "</HEAD>";
                $content .= start_poste($config, "wol", $machine);
                if (delete_dhcp_reservation($config, $machine['cn'])) {
                    $content .= "suppression de la reservation pour " . $machine['cn'] . "OK<br>\n";
                    $content .= start_poste($config, "reboot", $machine);
                } else {
                    $content .= "suppression de la reservation pour" . $machine['cn'] . "impossible<br>\n";
                }
            }
            break;
    }
    $html .= $content . "<br>\n";
    $html .= "<br><center>";
    $html .= "<a href=\"$retour.php?parc=$parc&action_poste=check\">Retour</a>";
    $html .= "</center>";
}
